<?php
namespace ElegantModules\ElegantCatalogs\Controllers;
use \ElegantModules\ElegantCatalogs\ElegantCatalogs as ElegantCatalogs;

class SettingsCtrl {

	private $settings_hook;

	public function __construct() {
		add_action( 'admin_menu', array( $this, 'add_menu_page' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'settings_page_scripts' ) );

		add_action( 'init', array( $this, 'connect_store' ) );
		add_action( 'wp_ajax_elegant_catalogs_disconnect_store', array( $this, 'disconnect_store' ) );

		// It would be nice if WooCommerce allowed for hooking into the WC_Admin_API_Keys->remove_key() method
		// This would let us automatically disconnect a store when keys are removed
		// Maybe one day we'll put that here
	}

	/**
	 * Register the submenu page.
	 *
	 * @return void
	 */
	public function add_menu_page() {
		
		$this->settings_hook = add_submenu_page(
			'woocommerce',
			__( 'Elegant Catalogs', 'elegant-catalogs' ), 
			__( 'Elegant Catalogs', 'elegant-catalogs' ), 
			'manage_options', 
			'elegant-catalogs', 
			array( $this, 'render_settings_page' ), 
			59 
		);
	}

	/**
	 * Render the submenu page.
	 *
	 * @return void
	 */
	public function render_settings_page() {
		ElegantCatalogs::template( 'settings-page', get_defined_vars() );
	}

	/**
	 * Enqueue JS and CSS for our settings page.
	 *
	 * @param  string 	$hook 	Page hook
	 *
	 * @return void
	 */
	public function settings_page_scripts( $hook ) {
		if ( $this->settings_hook === $hook ) {
			wp_enqueue_style( 'elegant-catalogs-settings-css', ELEGANT_CATALOGS_PLUGIN_DIR_URL . 'assets/css/settings.css', array(), ELEGANT_CATALOGS_VERSION );
			wp_enqueue_script('elegant-catalogs-settings-js', ELEGANT_CATALOGS_PLUGIN_DIR_URL . 'assets/js/settings.js', array( 'jquery' ), ELEGANT_CATALOGS_VERSION, true );
			wp_localize_script( 'elegant-catalogs-settings-js', 'elegantCatalogsSettings', array(
				'confirmMsg' => __( 'Are you sure you want to disconnect your store? You will not be able to add or edit catalogs for this store.', 'elegant-catalogs' )
			) );
		}
	}

	/**
	 * AJAX request to connect a store.
	 *
	 * @return void
	 */
	public function connect_store() {

		if ( empty( $_GET['action'] ) || 'elegant_catalogs_connect' !== $_GET['action'] ) {
			return;
		}

		// Security check
		if ( ! isset( $_GET['_wpnonce'] ) || ! wp_verify_nonce( str_replace( '/', '', $_GET['_wpnonce'] ), 'elegant_catalogs_admin' ) ) {
			return;
		}

		// Check license status
		$license_key = sanitize_text_field( $_GET['license_key'] );

		// Check if keys were already created
		global $wpdb;
		if ( $key_id = get_option( 'elegant_catalogs_wc_api_keys_record' ) ) {
			// Get any existing keys
			$keys = $wpdb->get_row( "SELECT * FROM {$wpdb->prefix}woocommerce_api_keys WHERE key_id = {$key_id}", ARRAY_A );
			if ( ! empty( $keys->consumer_key ) && ! empty( $keys->consumer_secret ) ) {
				// We have valid keys
				$consumer_key = $keys['consumer_key'];
				$consumer_secret = $keys['consumer_secret'];
			} else {
				// Get rid of invalid keys
				$wpdb->delete( 
					$wpdb->prefix . 'woocommerce_api_keys', 
					array( 'key_id' => $key_id ), 
					array( '%d' ) 
				);

				// Generate new keys
				$keys = $this->generate_keys();
				$consumer_key = $keys['consumer_key'];
				$consumer_secret = $keys['consumer_secret'];
			}

		} else {
			// Generate new keys
			$keys = $this->generate_keys();
			$consumer_key = $keys['consumer_key'];
			$consumer_secret = $keys['consumer_secret'];
		}

		// Send data to Elegant Catalogs
		$response = wp_remote_post( 
			trailingslashit( ELEGANT_CATALOGS_ORIGIN_URL ) . "wp-json/emapi/v1/catalogs/license/{$license_key}", 
			array(
				'body' => array(
					'state' => sanitize_text_field( $_GET['state'] ),
					'uuid' => sanitize_text_field( $_GET['uuid'] ),
					'store_name' => get_option( 'blogname' ),
					'store_url' => home_url(),
					'license' => $license_key,
					'consumer_key' => $consumer_key,
					'consumer_secret' => $consumer_secret,
				)
			) 
		);
		$body = json_decode( wp_remote_retrieve_body( $response ), true );

		// Store connection status
		if ( 'connected' === $body['data']['status'] ) {
			update_option( 'elegant_catalogs_site_connected', true );
			update_option( 'elegant_catalogs_license_key', $license_key );
			update_option( 'elegant_catalogs_store_id', (int) $body['data']['store_id'] );
		}

		wp_redirect( add_query_arg( 'page', 'elegant-catalogs', admin_url( 'admin.php' ) ) );
		exit;
	}

	/**
	 * AJAX request to disconnect a store.
	 *
	 * @return void
	 */
	public function disconnect_store() {

		// Security check
		if ( ! isset( $_POST['_wpnonce'] ) || ! wp_verify_nonce( $_POST['_wpnonce'], 'elegant_catalogs_admin' ) ) {
			return;
		}

		$license_key = sanitize_text_field( $_POST['license_key'] );

		// Send data to Elegant Catalogs
		$response = wp_remote_post( 
			trailingslashit( ELEGANT_CATALOGS_ORIGIN_URL ) . "wp-json/emapi/v1/catalogs/license/{$license_key}", 
			array(
				'body' => array(
					'action' => 'remove',
					'store_id' => get_option( 'elegant_catalogs_store_id' ),
				)
			) 
		);
		$body = json_decode( wp_remote_retrieve_body( $response ), true );

		// Store connection status
		if ( 'disconnected' === $body['data']['status'] ) {
			
			// Remove the API keys
			global $wpdb;
			$wpdb->delete(
				$wpdb->prefix . 'woocommerce_api_keys',
				array(
					'key_id' => (int) get_option( 'elegant_catalogs_wc_api_keys_record' )
				),
				array(
					'%d'
				)
			);

			delete_option( 'elegant_catalogs_site_connected' );
			delete_option( 'elegant_catalogs_license_key' );
			delete_option( 'elegant_catalogs_store_id' );
			delete_option( 'elegant_catalogs_wc_api_keys_record' );
		}

		wp_send_json_success( array(
			'status' => $body['data']['status']
		) );
	}

	/**
	 * Generates WooCommerce API keys.
	 *
	 * @return array
	 */
	public function generate_keys() {

		global $wpdb;
		$consumer_key    = 'ck_' . wc_rand_hash();
		$consumer_secret = 'cs_' . wc_rand_hash();
		$data = array(
			'user_id'         => get_current_user_id(),
			'description'     => __( 'Elegant Catalogs Catalog Builder', 'elegant-catalogs' ),
			'permissions'     => 'read',
			'consumer_key'    => wc_api_hash( $consumer_key ),
			'consumer_secret' => $consumer_secret,
			'truncated_key'   => substr( $consumer_key, -7 ),
		);

		$wpdb->insert(
			$wpdb->prefix . 'woocommerce_api_keys',
			$data,
			array(
				'%d',
				'%s',
				'%s',
				'%s',
				'%s',
				'%s',
			)
		);

		$response                    = $data;
		$response['consumer_key']    = $consumer_key;
		$response['consumer_secret'] = $consumer_secret;
		update_option( 'elegant_catalogs_wc_api_keys_record', $wpdb->insert_id );
		return $response;
	}
}